-- init

local NoNameNoFood = LibStub("AceAddon-3.0"):NewAddon("NoNameNoFood", "AceConsole-3.0", "AceComm-3.0", "AceSerializer-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local icon = LibStub("LibDBIcon-1.0")

-- utils

local NoNameNoFoodLDB = LibStub("LibDataBroker-1.1"):NewDataObject("NoNameNoFood", {
	type = "data source",
	text = "NoNameNoFood",
	icon = "Interface\\Icons\\INV_Misc_Food_15",
	OnTooltipShow = function(tooltip) tooltip:AddLine("NoNameNoFood") end,
	OnClick = function() 
		NoNameNoFood:CreateNNNFGUI()
		end,
})

-- functions

function NoNameNoFood:OnSelectedFood(widget, event, value, frame)
	local foodId = string.sub(value,3)

	if string.len(foodId) < 6 then return end

	local rankOne = ""
	local rankTwo = ""
	local rankThree = ""

	for k,v in pairs(self.db.global.recipes) do
		local currentPlayer = k
		for i,j in ipairs(v) do
			if j == NoNameNoFoodSpells[foodId][1] then rankOne = rankOne .. currentPlayer .. ", " end
			if j == NoNameNoFoodSpells[foodId][2] then rankTwo = rankTwo .. currentPlayer .. ", " end
			if j == NoNameNoFoodSpells[foodId][3] then rankThree = rankThree .. currentPlayer .. ", " end
		end
	end


	rankOne = string.sub(rankOne, 1, string.len(rankOne)-2)
	rankTwo = string.sub(rankTwo, 1, string.len(rankTwo)-2)
	rankThree = string.sub(rankThree, 1, string.len(rankThree)-2)

	frame:ReleaseChildren()

	local rankOneLink = GetSpellLink(NoNameNoFoodSpells[foodId][1])
	local rankOneIcon = AceGUI:Create("Icon")
	rankOneIcon:SetImage(GetItemIcon(foodId))
	rankOneIcon:SetImageSize(30,30)
	rankOneIcon:SetCallback("OnEnter", function(widget)
			GameTooltip:SetOwner(widget.frame);
			GameTooltip:SetHyperlink(rankOneLink);
			GameTooltip:Show();
		end)
	rankOneIcon:SetCallback("OnLeave", function(widget) GameTooltip:FadeOut() end)

	local rankOneLabel = AceGUI:Create("Label")
	rankOneLabel:SetText("* ".. rankOne)

	local rankTwoLink = GetSpellLink(NoNameNoFoodSpells[foodId][2])
	local rankTwoIcon = AceGUI:Create("Icon")
	rankTwoIcon:SetImage(GetItemIcon(foodId))
	rankTwoIcon:SetImageSize(30,30)
	rankTwoIcon:SetCallback("OnEnter", function(widget)
			GameTooltip:SetOwner(widget.frame);
			GameTooltip:SetHyperlink(rankTwoLink);
			GameTooltip:Show();
		end)
	rankTwoIcon:SetCallback("OnLeave", function(widget) GameTooltip:FadeOut() end)

	local rankTwoLabel = AceGUI:Create("Label")
	rankTwoLabel:SetText("** ".. rankTwo)

	local rankThreeLink = GetSpellLink(NoNameNoFoodSpells[foodId][3])
	local rankThreeIcon = AceGUI:Create("Icon")
	rankThreeIcon:SetImage(GetItemIcon(foodId))
	rankThreeIcon:SetImageSize(30,30)
	rankThreeIcon:SetCallback("OnEnter", function(widget) 
			GameTooltip:SetOwner(widget.frame);
			GameTooltip:SetHyperlink(rankThreeLink);
			GameTooltip:Show();
		end)
	rankThreeIcon:SetCallback("OnLeave", function(widget) GameTooltip:FadeOut() end)

	local rankThreeLabel = AceGUI:Create("Label")
	rankThreeLabel:SetText("*** ".. rankThree)

	local rankOneFrame = AceGUI:Create("SimpleGroup")
	rankOneFrame:SetFullWidth(true)
	rankOneFrame:SetLayout("Flow")
	rankOneFrame:AddChild(rankOneIcon)
	rankOneFrame:AddChild(rankOneLabel)

	local rankTwoFrame = AceGUI:Create("SimpleGroup")
	rankTwoFrame:SetFullWidth(true)
	rankTwoFrame:SetLayout("Flow")
	rankTwoFrame:AddChild(rankTwoIcon)
	rankTwoFrame:AddChild(rankTwoLabel)

	local rankThreeFrame = AceGUI:Create("SimpleGroup")
	rankThreeFrame:SetFullWidth(true)
	rankThreeFrame:SetLayout("Flow")
	rankThreeFrame:AddChild(rankThreeIcon)
	rankThreeFrame:AddChild(rankThreeLabel)

	frame:AddChild(rankOneFrame)
	frame:AddChild(rankTwoFrame)
	frame:AddChild(rankThreeFrame)
end

function NoNameNoFood:ListSelfRecipes()
	local selfRecipes = {}
	for k,v in pairs(NoNameNoFoodSpells) do
		for i=1,3 do
			if IsPlayerSpell(v[i]) then
				table.insert(selfRecipes, v[i])
			end
		end
	end
	return selfRecipes
end

function NoNameNoFood:PushWelcomeUpdate(selfRecipes)
	NoNameNoFood:SendCommMessage("nnnf_welcome", self:Serialize(selfRecipes), "GUILD")
end

function NoNameNoFood:OnWelcomeUpdate(prefix, message, distribution, sender)
	local success, msg = self:Deserialize(message)
	self.db.global.recipes[sender] = msg
	if sender ~= UnitName("player") then NoNameNoFood:SendCommMessage("nnnf_whisper", self:Serialize(NoNameNoFood:ListSelfRecipes()), "WHISPER", sender) end
end

function NoNameNoFood:OnWhisperUpdate(prefix, message, distribution, sender)
	if sender == UnitName("player") then return end
	local success, msg = self:Deserialize(message)
	self.db.global.recipes[sender] = msg
end

-- init

function NoNameNoFood:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("NoNameNoFoodDB" , {
		global = {
				recipes = {}
			}
		})

	NoNameNoFood:RegisterChatCommand("nnnf", "CreateNNNFGUI")

	-- register ace3-comm msg
	NoNameNoFood:RegisterComm("nnnf_whisper", function(prefix, message, distribution, sender) NoNameNoFood:OnWhisperUpdate(prefix, message, distribution, sender) end)
	NoNameNoFood:RegisterComm("nnnf_welcome", function(prefix, message, distribution, sender) NoNameNoFood:OnWelcomeUpdate(prefix, message, distribution, sender) end)
	
	-- send own welcome
	NoNameNoFood:PushWelcomeUpdate(selfRecipes)

	self.db.profile = {
			minimap = {
				hide = false,
			},
		}

	icon:Register("NoNameNoFood", NoNameNoFoodLDB, self.db.profile.minimap)
end

function NoNameNoFood:CreateNNNFGUI(input)
	
	self.db.global.recipes[UnitName("Player")] = NoNameNoFood:ListSelfRecipes()

	local frame = AceGUI:Create("Frame")
	frame:SetTitle("NoNameNoFood")
	frame:SetStatusText("version beta 1.1")
	frame:SetCallback("OnClose", function(widget) AceGUI:Release(widget) end)
	frame:SetLayout("Fill")

	local treewidget = AceGUI:Create("TreeGroup")
	treewidget:SetTree(NoNameNoFoodRecipes)
	treewidget:SetFullWidth(true)
	treewidget:SetFullHeight(true)
	treewidget:SetLayout("Fill")
	frame:AddChild(treewidget)

	local recipeContainer = AceGUI:Create("SimpleGroup")
	recipeContainer:SetFullWidth(true)
	recipeContainer:SetFullHeight(true)
	recipeContainer:SetLayout("Fill")	
	treewidget:AddChild(recipeContainer)

	local recipeScroll = AceGUI:Create("ScrollFrame")
	recipeScroll:SetLayout("List")
	recipeScroll:SetFullWidth(true)
	recipeScroll:SetFullHeight(true)
	recipeContainer:AddChild(recipeScroll)

	treewidget:SetCallback("OnGroupSelected", function(widget, event, value, frame) NoNameNoFood:OnSelectedFood(widget, event, value, recipeScroll) end)
end

